/*-
 * Copyright 2008 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_ANALOG_DRIVER_FACTORY_H__
#define __CX88_ANALOG_DRIVER_FACTORY_H__

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <new>
#include "cx88_analog_driver.h"

namespace cx88_analog_driver_factory
{

   typedef cx88_analog_driver* (*constructFn) (const char*,
                                               cx88_xml_config&,
                                               tuner_config&,
                                               const char*,
                                               capvec&,
                                               cx88_analog_driver::cx88_analog_mode,
                                               cx88_analog_driver*,
                                               cx88_analog_driver*,
                                               int&);

   void registerDriver(const char *name, constructFn ctor);

   template <typename driverT>
   cx88_analog_driver *construct(
      const char *name,
      cx88_xml_config &cx88conf,
      tuner_config &tunerconf,
      const char *devnode,
      capvec &captures,
      cx88_analog_driver::cx88_analog_mode mode,
      cx88_analog_driver *slave,
      cx88_analog_driver *pip_driver,
      int &error)
   {
      return new(std::nothrow) driverT(name, cx88conf, tunerconf, devnode, captures, mode, slave, pip_driver, error);
   }

#define CX88_REGISTER_ANALOG_DRIVER(class)                                                                                              \
   struct class##_analog_registrar                                                                                                      \
   {                                                                                                                                    \
      class##_analog_registrar() { cx88_analog_driver_factory::registerDriver(#class, &cx88_analog_driver_factory::construct<class>); } \
   };                                                                                                                                   \
   static class##_analog_registrar register_analog_##class;


   cx88_analog_driver *create(
      cx88_xml_config &cx88conf,
      tuner_config &tunerconf,
      const char *devnode,
      capvec &captures,
      cx88_analog_driver::cx88_analog_mode mode,
      cx88_analog_driver *slave,
      cx88_analog_driver *pip_driver,
      int &error);
   
};

#endif
