/*-
 * Copyright 2008 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_CAPTURE_FACTORY_H__
#define __CX88_CAPTURE_FACTORY_H__

#include <new>
#include "tuner_config.h"
#include "cx88_xml_config.h"
#include "cx88_capture.h"

namespace cx88_capture_factory
{

   typedef cx88_capture* (*build_capture_func)(std::string&, const char*, tuner_config&, int&);

   void registerCapture(const char *name, build_capture_func builder);

   template <typename captureT>
   cx88_capture *construct(std::string &id, const char *name, tuner_config &options, int &error)
   {
      return new(std::nothrow) captureT(id, name, options, error);
   }

#define CX88_REGISTER_CAPTURE(class)                                                                                  \
   struct class##_registrar                                                                                           \
   {                                                                                                                  \
      class##_registrar() { cx88_capture_factory::registerCapture(#class, &cx88_capture_factory::construct<class>); } \
   };                                                                                                                 \
   static class##_registrar register_##class;

   void init(cx88_xml_config &conf, int &error);

   cx88_capture *create(const char *url, tuner_config &options, int &error);
};

#endif
