/*-
 * Copyright 2012 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <sys/errno.h>
#include <stdio.h>
#include "cx88_dvb_hal.h"

cx88_dvb_hal::cx88_dvb_hal(void)
   : m_path(NULL)
{}

cx88_dvb_hal::~cx88_dvb_hal(void)
{
   stop();
   free(m_path);
}

int cx88_dvb_hal::start(const char *adapterName, const char *nodeName)
{
   char *devnode = NULL;
   static const char *caps[] = {"dvb"};
   asprintf(&devnode, "/dev/dvb/%s/%s", adapterName, nodeName);
   asprintf(&m_path, CX88_HAL_DEVICE_PATH "/cx88_dvb_%s_%s", adapterName, nodeName);
   if ((m_path == NULL) || (devnode == NULL))
   {
      free(m_path);
      m_path = NULL;
      free(devnode);
      return ENOMEM;
   }
   cx88_hal_property props[4];
   props[0].name = "dvb.device";
   props[0].value = devnode;
   props[1].name = "info.category";
   props[1].value = "dvb";
   props[2].name = "info.product";
   props[2].value = "DVB Device";
   props[3].name = "info.subsystem";
   props[3].value = "dvb";
   int error = cx88_hal::start(m_path, props, sizeof(props) / sizeof(props[0]), caps, sizeof(caps) / sizeof(caps[0]));   
   free(devnode);
   return error;
}

