/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_RADIO_CAPTURE_H__
#define __CX88_RADIO_CAPTURE_H__

#include <pthread.h>
#include <linux/videodev2.h>
extern "C"
{
   #include <cuse4bsd.h>
}
#include "cx88_capture.h"
#include "tuner_config.h"
#include "cx88_analog_driver.h"

class cx88_radio_capture
   : public cx88_capture
{

   public:

      cx88_radio_capture(const std::string &id, const char *devnode, tuner_config &options, int &error);

      virtual ~cx88_radio_capture(void);

      virtual int set_driver(cx88_driver *driver);

   protected:

      enum
      {
         CX88_RADIO_CAPTURE_UNINITIALIZED,
         CX88_RADIO_CAPTURE_LOCK_CREATED,
         CX88_RADIO_CAPTURE_CV_CREATED,
         CX88_RADIO_CAPTURE_CUSE_INITIALIZED,
         CX88_RADIO_CAPTURE_THREAD_CREATED
      } m_state;

      static const struct cuse_methods m_cuse_methods;
      struct cuse_dev *m_cuse_dev;
      pthread_mutex_t m_lock;
      pthread_cond_t m_event;
      pthread_t m_cuse_thread;
      cx88_analog_driver *m_driver;
      struct v4l2_tuner m_tuner;
      struct v4l2_capability m_caps;
      struct v4l2_audio m_audio;
      uint32_t m_frequency;
      int m_channel_status;
      bool m_opened;

      int ioctl(int fflags, unsigned long code, void *user_data);

      virtual void process_buffer(void *data, size_t num_bytes, size_t packet_size = 0) {}

      virtual void process_notification(uint32_t events, int error);

      static void *cuse_thread_routine(void *arg);

      static int radio_open(struct cuse_dev *dev, int fflags);

      static int radio_close(struct cuse_dev *dev, int fflags);

      static int radio_ioctl(struct cuse_dev *dev, int fflags, unsigned long cmd, void *user_data);

};

#endif

